import forms
from views import render
from django.http import HttpRequest, HttpResponse, HttpResponseRedirect, HttpResponseForbidden
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from decorators import handle404, check_wiki_installed

@check_wiki_installed
@login_required
def create_featured (request):
    if not request.user.is_staff:
        return HttpResponseForbidden()    
    if request.method == 'POST':
        featured_page_form = forms.MakeFeatured(request.POST)
        if featured_page_form.is_valid():
            featured_page_form.save()
            return HttpResponseRedirect('.')    
    elif request.method == 'GET':
        featured_page_form = forms.MakeFeatured()
    payload = {'form':featured_page_form}
    return render(request, 'djikiki/edit.html', payload)
        
@check_wiki_installed        
@login_required
def user_list (request):
    if not request.user.is_staff:
        return HttpResponseForbidden()
    user_list = User.objects.all()
    payload = {'user_list':user_list}
    return render(request, 'djikiki/userlist.html', payload)

@login_required
def install (request):
    if not request.user.is_staff:
        return HttpResponseForbidden()
    if request.method == 'POST' :
        install_form = forms.InstallForm(request.POST)
        if install_form.is_valid():
            install_form.save()
            return HttpResponseRedirect('/')
    elif request.method == 'GET':
        install_form = forms.InstallForm()
    payload = {'form':install_form}
    return render(request, 'djikiki/edit.html', payload)        
        
    
        
    


from django import forms
import re
from django.forms import ValidationError
from django.contrib.auth.models import User
from django.contrib.auth import authenticate
from django.contrib.sites.models import Site
from django.template import Context, loader
from django.utils.translation import ugettext as _

class UserCreationForm(forms.Form):
    """A form that creates a user, with no privileges, from the given username and password."""
    username = forms.CharField(max_length = 30, required = True)
    password1 = forms.CharField(max_length = 30, required = True, widget = forms.PasswordInput)
    password2 = forms.CharField(max_length = 30, required = True, widget = forms.PasswordInput)

    def clean_username (self):
        alnum_re = re.compile(r'^\w+$')
        if not alnum_re.search(self.cleaned_data['username']):
            raise ValidationError("This value must contain only letters, numbers and underscores.")
        self.isValidUsername()
        return self.cleaned_data['username']

    def clean (self):
        if self.cleaned_data['password1'] != self.cleaned_data['password2']:
            raise ValidationError(_("The two password fields didn't match."))
        return super(forms.Form, self).clean()
        
    def isValidUsername(self):
        try:
            User.objects.get(username=self.cleaned_data['username'])
        except User.DoesNotExist:
            return
        raise ValidationError(_('A user with that username already exists.'))

class AuthenticationForm(forms.Form):
    """
    Base class for authenticating users. Extend this to get a form that accepts
    username/password logins.
    """
    username = forms.CharField(required = True, max_length = 30)
    password = forms.CharField(required = True, max_length = 30, widget = forms.PasswordInput)
    def set_request (self, request):
        self.request = request
        
        
    def clean (self):
        user = authenticate(username=self.cleaned_data['username'], password=self.cleaned_data['password'])
	if self.request and not self.request.session.test_cookie_worked():
	    raise ValidationError(_("Your Web browser doesn't appear to have cookies enabled. Cookies are required for logging in."))
        if user is None:
            raise ValidationError(_("Please enter a correct username and password. Note that both fields are case-sensitive."))
        elif not user.is_active:
            raise ValidationError(_("This account is inactive."))
        else:
            self.user = user
        return super(forms.Form, self).clean()

class PasswordResetForm(forms.Form):
    """A form that lets a user request a password reset"""
    email = forms.EmailField(required = True)

    def clean_email (self):
        try:
           self.user = User.objects.get(email__iexact=self.cleaned_data['email'])
        except User.DoesNotExist:
            print '***'
            raise ValidationError(_("That e-mail address doesn't have an associated user account. Are you sure you've registered?"))
        return self.cleaned_data['email']

    def save(self, domain_override=None, email_template_name='registration/password_reset_email.html'):
        "Calculates a new password randomly and sends it to the user"
        from django.core.mail import send_mail
        user = self.user
        new_pass = User.objects.make_random_password()
        user.set_password(new_pass)
        user.save()
        if not domain_override:
            current_site = Site.objects.get_current()
            site_name = current_site.name
            domain = current_site.domain
        else:
            site_name = domain = domain_override
        t = loader.get_template(email_template_name)
        c = {
        'new_password': new_pass,
        'email': user.email,
        'domain': domain,
        'site_name': site_name,
        'user': user,
        }
        send_mail(_('Password reset on %s') % site_name, t.render(Context(c)), None, [user.email])

class PasswordChangeForm(forms.Form):
    """A form that lets a user change his password."""
    old_password = forms.CharField(widget=forms.PasswordInput, required = True, max_length = 30)
    new_password1 = forms.CharField(widget=forms.PasswordInput, required = True, max_length = 30)
    new_password2 = forms.CharField(widget=forms.PasswordInput, required = True, max_length = 30)

    def set_user (self, user):
        self.user = user

    def clean_old_password (self):
        if not self.user.check_password(self.cleaned_data['old_password']):
            raise ValidationError(_("Your old password was entered incorrectly. Please enter it again."))
        return self.cleaned_data['old_password']

    def clean (self):
         if self.cleaned_data['new_password1'] != self.cleaned_data['new_password2']:
             raise ValidationError(_("The two new password fields didn't match."))
         return super(forms.Form, self).clean()

    def save (self, new_password):
        "Saves the new password."
        self.user.set_password(new_password)
        self.user.save()

class AdminPasswordChangeForm(forms.Form):
    """A form used to change the password of a user in the admin interface."""
    password1 = forms.CharField(widget = forms.PasswordInput, max_length = 30, required = True)
    password2 = forms.CharField(widget = forms.PasswordInput, max_length = 30, required = True)	
    
    def set_user (self, user):
        self.user = user
    	
    def clean (self):
        if self.cleaned_data['password1'] != self.cleaned_data['password2']:
            raise ValidationError(_("The two password fields didn't match."))
        return super(forms.Form, self).clean()

    def save(self):
        self.user.set_password(self.cleaned_data['password1'])
        self.user.save()      
      


import views
import userforms as forms
from django.shortcuts import render_to_response, get_object_or_404
from django.http import HttpResponse, HttpResponseRedirect
from django.contrib.auth.models import User
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.template import RequestContext
from django.contrib.sites.models import Site, RequestSite
from django.utils.translation import ugettext as _
from django.contrib.auth.decorators import login_required
from django.core.exceptions import PermissionDenied


def user_creation (request, template_name='registration/create_user.html'):
    if request.method == 'POST':
       user_creation_form = forms.UserCreationForm(request.POST)
       if user_creation_form.is_valid():
          username = user_creation_form.cleaned_data['username']
          password = user_creation_form.cleaned_data['password1']
          User.objects.create_user(username, '', password)
          return HttpResponseRedirect('/accounts/login/')
    elif request.method == 'GET':
       user_creation_form = forms.UserCreationForm()
    return render_to_response(template_name, {'form':user_creation_form}, context_instance=RequestContext(request))

def login(request, template_name='registration/login.html', redirect_field_name=REDIRECT_FIELD_NAME):
    """Displays the login form and handles the login action."""
    redirect_to = request.REQUEST.get(redirect_field_name, '')
    if Site._meta.installed:
        current_site = Site.objects.get_current()
    else:
        current_site = RequestSite(request)
    if request.method == 'POST':
       login_form = forms.AuthenticationForm(request.POST)
       login_form.set_request(request)
       if login_form.is_valid():
            # Light security check -- make sure redirect_to isn't garbage.
            if not redirect_to or '//' in redirect_to or ' ' in redirect_to:
                from django.conf import settings
                redirect_to = settings.LOGIN_REDIRECT_URL
            from django.contrib.auth import login
            user = login_form.user
            login(request, user)
            request.session.delete_test_cookie()
            return HttpResponseRedirect(redirect_to)
    elif request.method == 'GET':
        login_form = forms.AuthenticationForm()
    request.session.set_test_cookie()
    return render_to_response(template_name, {
        'form': login_form,
        redirect_field_name: redirect_to,
        'site_name': current_site.name,
    }, context_instance=RequestContext(request))

def logout(request, next_page=None, template_name='registration/logged_out.html'):
    "Logs out the user and displays 'You are logged out' message."
    from django.contrib.auth import logout
    logout(request)
    if next_page is None:
        return render_to_response(template_name, {'title': _('Logged out')}, context_instance=RequestContext(request))
    else:
        # Redirect to this page until the session has been cleared.
        return HttpResponseRedirect(next_page or request.path)

def logout_then_login(request, login_url=None):
    "Logs out the user if he is logged in. Then redirects to the log-in page."
    if not login_url:
        from django.conf import settings
        login_url = settings.LOGIN_URL
    return logout(request, login_url)

def redirect_to_login(next, login_url=None, redirect_field_name=REDIRECT_FIELD_NAME):
    "Redirects the user to the login page, passing the given 'next' page"
    if not login_url:
        from django.conf import settings
        login_url = settings.LOGIN_URL
    return HttpResponseRedirect('%s?%s=%s' % (login_url, redirect_field_name, next))

def password_reset(request, is_admin_site=False, template_name='registration/password_reset_form.html',
        email_template_name='registration/password_reset_email.html'):
    if request.method == 'POST':
       form = forms.PasswordResetForm(request.POST)
       if form.is_valid():
            if is_admin_site:
                form.save(domain_override=request.META['HTTP_HOST'])
            else:
                form.save(email_template_name=email_template_name)
            return HttpResponseRedirect('%sdone/' % request.path)       
    elif request.method == 'GET':
        form = forms.PasswordResetForm()
    print template_name
    return render_to_response(template_name, {'form': form}, context_instance=RequestContext(request))

def index(request):
    return render_to_response('registration/password_reset_form.html', {}, context_instance=RequestContext(request))

def password_change(request, template_name='registration/password_change_form.html'):
    if request.method == 'POST':
        form = forms.PasswordChangeForm(request.POST)
        form.set_user(request.user)
        if form.is_valid():
            form.save(form.cleaned_data['new_password1'])
            return HttpResponseRedirect('%sdone/' % request.path)       
    if request.method == 'GET':
        form = forms.PasswordChangeForm()
    return render_to_response(template_name, {'form': form},
        context_instance=RequestContext(request))
password_change = login_required(password_change)

def user_change_password(request, id):
    if not request.user.has_perm('auth.change_user'):
        raise PermissionDenied    
    user = get_object_or_404(User, pk=id)
    if request.method == 'POST':
        form = forms.AdminPasswordChangeForm(request.POST)
        form.set_user(user)
        if form.is_valid():
            new_user = form.save()
            msg = _('Password changed successfully.')
            request.user.message_set.create(message=msg)
            return HttpResponseRedirect('..')
    elif request.method == 'GET':
        form = forms.AdminPasswordChangeForm()
    return render_to_response('admin/auth/user/change_password.html', {'title': _('Change password: %s') % user.username, 'form': form, 'is_popup': '_popup' in request.REQUEST, 'add': True, 'change': False, 'has_delete_permission': False, 'has_change_permission': True, 'has_absolute_url': False, 'opts': User._meta, 'original': user, 'show_save': True, }, context_instance=RequestContext(request))

@login_required
def profile (request):
    return views.user_account(request, request.user.username)


from django.shortcuts import render_to_response
from django.http import HttpRequest, HttpResponse, HttpResponseRedirect, HttpResponseForbidden, HttpResponseNotFound
from django.contrib.auth.decorators import login_required
from django.template import RequestContext
from django.contrib.auth.models import User
from django.core.paginator import Paginator
import random

import forms
import djikikisettings
from decorators import handle404, check_wiki_installed
from models import *


@check_wiki_installed
def index (request):
    featured_pages = FeaturedPage.objects.all().order_by('-ordering')
    try:
       latest_featured = featured_pages[0].page
    except IndexError:
       latest_featured = ''
    recently_featured = featured_pages[:djikikisettings.recently_featured]
    payload = {'latest_featured':latest_featured, 'recently_featured':recently_featured}
    return render(request, 'djikiki/index.html', payload)

@check_wiki_installed
@handle404
def detail (request, slug, mode = 'article'):
    page = Page.objects.get(slug = slug)
    categories = [category for category in page.current_revision.categories.all()]
    if mode == 'article':
        payload = {'page':page, 'categories':categories,}
        return render(request, 'djikiki/detail.html', payload)
    elif mode == 'discuss':    
        payload = {'page':page, 'categories':categories, }
        return render(request, 'djikiki/discuss.html', payload)

@check_wiki_installed
@handle404
def category (request, text):
    try:
       page_num = request.GET['page']
       page_num = int(page_num)
    except:
       page_num = 1    
    category = Category.objects.get(text = text)
    page_revs = category.pagerevision_set.all()
    pages = Page.objects.filter(current_revision__in = page_revs)
    pages_ = Paginator(pages, djikikisettings.details_per_page)
    next_page = page_num + 1
    prev_page = page_num - 1
    pages = pages_.page(page_num)
    has_next = pages.has_next()
    has_previous = pages.has_previous()    
    payload = {'category':category, 'pages':pages.object_list,  'has_next':has_next, 'has_previous':has_previous, 'next_page':next_page, 'prev_page':prev_page }
    return render(request, 'djikiki/category.html', payload)

@check_wiki_installed
@handle404
def revisions (request, slug, mode='article'):
    page = Page.objects.get(slug = slug)
    if mode == 'article':
        is_discussion = False
        page_revisions = PageRevision.objects.filter(revision_for = page, is_discussion = is_discussion).order_by('-revision_num')
    else:
        is_discussion = True
        page_revisions = PageRevision.objects.filter(revision_for = page, is_discussion = is_discussion).order_by('-revision_num')
    try:
       page_num = request.GET['page']
       page_num = int(page_num)
    except:
       page_num = 1
    page_revisions_ = Paginator(page_revisions, djikikisettings.details_per_page)
    next_page = page_num + 1
    prev_page = page_num - 1
    page_revisions = page_revisions_.page(page_num)
    has_next = page_revisions.has_next()
    has_previous = page_revisions.has_previous()    
    payload = {'page':page, 'revisions':page_revisions.object_list, 'has_next':has_next, 'has_previous':has_previous, 'next_page':next_page, 'prev_page':prev_page }
    return render(request, 'djikiki/revisions.html', payload)   

@check_wiki_installed
@handle404
def old_page (request, id):
    page_revision = PageRevision.objects.get(id = id)
    categories = [category for category in page_revision.categories.all()]
    payload = {'page_revision':page_revision, 'categories':categories}
    return render(request, 'djikiki/oldpage.html', payload)

def random_page (request):
    page_count = Page.objects.all().count()
    random_page_id = random.randint(1, page_count)
    page = Page.objects.get(id = random_page_id)
    title = '/page/%s/' % page.slug
    return HttpResponseRedirect(title)

def recently_featured (request):
    import pdb
    #pdb.set_trace()
    featured = FeaturedPage.objects.all().order_by('-ordering')
    paged_featured = Paginator(featured, djikikisettings.details_per_page)
    try:
       page_num = request.GET['page']
       page_num = int(page_num)
    except:
       page_num = 1
    next_page = page_num + 1
    prev_page = page_num - 1
    featured = paged_featured.page(page_num)
    has_next = featured.has_next()
    has_previous = featured.has_previous()
    payload = {'featured':featured.object_list, 'has_next':has_next, 'has_previous':has_previous, 'next_page':next_page, 'prev_page':prev_page}
    return render(request, 'djikiki/featured.html', payload)    

@check_wiki_installed
@login_required    
def create (request):
    if request.method == 'POST':
        create_form = forms.CreatePage(request.POST)
        if create_form.is_valid():
            page = create_form.save(request.user)
            return HttpResponseRedirect(page.get_absolute_url())
    if request.method == 'GET':
        create_form = forms.CreatePage()
    payload = {'form':create_form}
    return render(request, 'djikiki/create.html', payload)      

@check_wiki_installed
@login_required
@handle404
def edit (request, slug, mode = 'article'):
    if request.method == 'POST':
        page = None
        edit_form = forms.EditPage(page, request.POST)
        if edit_form.is_valid():
            page = edit_form.save(slug, request.user)
            return HttpResponseRedirect(page.get_absolute_url())                
    if request.method == 'GET':
        page = Page.objects.get(slug = slug)
        edit_form = forms.EditPage(page)
    payload = {'form':edit_form, 'page':page}
    return render(request, 'djikiki/edit.html', payload)

@check_wiki_installed
@login_required
@handle404
def discuss_edit (request, slug):
    if request.method == 'POST':
        page = None
        edit_form = forms.DiscussEditPage(page, request.POST)
        if edit_form.is_valid():
            page = edit_form.discuss_save(slug, request.user)
            return HttpResponseRedirect(page.get_discuss_url())         
    if request.method == 'GET':
        page = Page.objects.get(slug = slug)
        edit_form = forms.DiscussEditPage(page)
    payload = {'form':edit_form, 'page':page}
    return render(request, 'djikiki/edit.html', payload)

@check_wiki_installed
@handle404
def user_account (request, username):
    user_for = User.objects.get(username = username)
    user_edits = PageRevision.objects.filter(user = user_for)
    user_create = Page.objects.filter(user = user_for)
    payload = {'user_edits':user_edits, 'user_create':user_create,'user_for':user_for}
    return render(request, 'djikiki/useredits.html', payload)   

def render (request, template, payload):
    recent_pages = Page.objects.all().order_by('-created_on')[:5]
    recent_changes = Page.objects.all().order_by('-modified_on')[:5]
    try:
        wiki = Wiki.objects.latest()
    except:
        wiki = ''
    payload.update({'wiki':wiki})
    payload.update({'recent_pages':recent_pages, 'recent_changes':recent_changes})
    return render_to_response(template, payload, RequestContext(request))

def administer (request):
    """Provides administaration functions"""
    pass
        

     


from django import forms
import models
from datetime import datetime

class WideTextArea(forms.Textarea):
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('attrs',{}).update({'rows': '20', 'cols':'80'})
        super(WideTextArea, self).__init__(*args, **kwargs)

class CreatePage(forms.Form):
    title = forms.CharField()
    text = forms.CharField(widget = WideTextArea)
    edit_summary = forms.CharField()
    categories = forms.CharField(required = False)

    def save(self, user):
        page = models.Page(title = self.cleaned_data['title'], user = user)
        page.save()
        page_revision = models.PageRevision(wiki_text = self.cleaned_data['text'], edit_summary = self.cleaned_data['edit_summary'], user = user, revision_for = page)
        page_revision.save()
        discuss_revision = models.PageRevision(wiki_text = 'Auto created first revision.', edit_summary = '', user = user, revision_for = page, is_discussion = True)
        discuss_revision.save()
        categories_list = self.cleaned_data['categories'].split()
        for category in categories_list:
            try:
                cat_ = models.Category.objects.get(text = category)
            except models.Category.DoesNotExist:
                cat_ = models.Category(text = category)
                cat_.save()
            page_revision.categories.add(cat_)
        page.current_revision = page_revision
        page.current_disc_rev = discuss_revision
        page.save()
        return page
    

class EditPage(forms.Form):
    text = forms.CharField(widget = WideTextArea)
    edit_summary = forms.CharField()
    categories = forms.CharField(required = False)
    
    def __init__(self, page_ = None, *args, **kwargs):
        super(forms.Form, self).__init__(*args, **kwargs)
        if not page_ is None:
            self.fields['text'].initial = page_.current_revision.wiki_text
            categories = ''
            for category in page_.current_revision.categories.all():
                categories += category.text
                categories += ' '        
            self.fields['categories'].initial = categories

    def save (self, slug, user):
        page = models.Page.objects.get(slug = slug)
        revision_count = models.PageRevision.objects.filter(revision_for = page, is_discussion = False).count()
        new_rev = revision_count + 1
        page_revision = models.PageRevision(wiki_text = self.cleaned_data['text'], edit_summary = self.cleaned_data['edit_summary'], user = user, revision_for = page, revision_num = new_rev)
        page_revision.save()
        categories_list = self.cleaned_data['categories'].split()
        for category in categories_list:
            try:
                cat_ = models.Category.objects.get(text = category)
            except models.Category.DoesNotExist:
                cat_ = models.Category(text = category)
                cat_.save()
            page_revision.categories.add(cat_)
        page.current_revision = page_revision
        page.modified_on = datetime.now()
        page.save()
        return page

class DiscussEditPage(forms.Form):
    text = forms.CharField(widget = WideTextArea)
    edit_summary = forms.CharField()

    def __init__(self, page_ = None, *args, **kwargs):
        super(forms.Form, self).__init__(*args, **kwargs)
        if not page_ is None:
            self.fields['text'].initial = page_.current_disc_rev.wiki_text
            
    def discuss_save (self, slug, user):
        page = models.Page.objects.get(slug = slug)
        revision_count = models.PageRevision.objects.filter(revision_for = page, is_discussion = True).count()
        new_rev = revision_count + 1
        page_revision = models.PageRevision(wiki_text = self.cleaned_data['text'], edit_summary = self.cleaned_data['edit_summary'], user = user, is_discussion = True, revision_for = page, revision_num = new_rev)
        page_revision.save()
        page.current_disc_rev = page_revision
        page.modified_on = datetime.now()
        page.save()
        return page

class MakeFeatured(forms.Form):
    title = forms.CharField()
    def save (self):
        page = models.Page.objects.get(title = self.cleaned_data['title'])
        fpage = models.FeaturedPage(page = page)
        fpage.save()

class InstallForm (forms.Form):
    """Installation data. Just the name of the wiki, really"""
    name = forms.CharField()
    tagline = forms.CharField()
    def save (self):
        wiki = models.Wiki(name = self.cleaned_data['name'], tagline = self.cleaned_data['tagline'])
        wiki.save()
        
        



        
        

        
        

        


from django.conf.urls.defaults import *
urlpatterns = patterns('djikiki.views',
    url(r'^$', 'index', name='djikiki_index'),
    url(r'^create/$', 'create', name='djikiki_create'),
    url(r'^page/(?P<slug>[^\.^/]+)/$', 'detail', name='djikiki_detail'),
    url(r'^edit/(?P<slug>[^\.^/]+)/$', 'edit', name='djikiki_edit'),
    url(r'^page/(?P<slug>[^\.^/]+)/revisions/$', 'revisions', name='djikiki_revisions'),
    url(r'^discuss/(?P<slug>[^\.^/]+)/$', 'detail', {'mode':'discuss'}, name='djikiki_discuss'),
    url(r'^discuss-edit/(?P<slug>[^\.^/]+)/$', 'discuss_edit', name='djikiki_discuss_edit'),
    url(r'^discuss/(?P<slug>[^\.^/]+)/revisions/$', 'revisions', {'mode':'discuss'}, name='djikiki_discuss_revisions'),
    url(r'^category/(?P<text>[^\.^/]+)/$', 'category', name='djikiki_category'),
    url(r'^revision/(?P<id>\d+)/$', 'old_page', name='djikiki_old_page'),
    url(r'^user-edit/(?P<username>\w+)/$', 'user_account', {}, name='djikiki_useredits'),
    url(r'^random/$', 'random_page', {}, name='djikiki_random'),
    url(r'^featured/$', 'recently_featured', {}, name='djikiki_featured'),

    )

urlpatterns += patterns('djikiki.adminviews',
    url(r'^administration/create-featured/$', 'create_featured', {}, name='djikiki_create_featured'),
    url(r'^administration/user-list/$', 'user_list', {}, name='djikiki_userlist'),
    url(r'^administration/install/$', 'install', {}, name='djikiki_install'),
   )

urlpatterns += patterns('djikiki.userviews',
    url(r'^profile/$', 'profile', name= 'djikiki_profile'),
   )





from django.conf.urls.defaults import *
from django.conf import settings

from django.contrib import admin

admin.autodiscover()

urlpatterns = patterns('',
    (r'^djikiki/', include('djikiki.urls')),
    (r'^accounts/', include('registration.urls')),

)

urlpatterns += patterns('',
     ('^admin/(.*)', admin.site.root),
    )

if settings.DEBUG:
    import os
    dirname = os.path.dirname(globals()["__file__"])
    media_dir = os.path.join(dirname, 'site_media')
    urlpatterns += patterns('',
        (r'^site_media/(?P<path>.*)$', 'django.views.static.serve', {'document_root': media_dir}),
        
        )




import ez_setup
ez_setup.use_setuptools()
from setuptools import setup, find_packages
setup(
    name = "djikiki",
    version = "0.4",
    packages = find_packages(),
    author = "Shabda Raaj",
    author_email = "shabda@uswaretech.com",
    description = "A django based wiki",
    url = "https://svn.uswaretech.com/djikiki/trunk/djikiki/",
    include_package_data = True
)



# Django settings for wiki project.
import os.path
from random import choice

from localsettings import *


TEMPLATE_DEBUG = DEBUG

ADMINS = (
    # ('Your Name', 'your_email@domain.com'),
)

MANAGERS = ADMINS


# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be avilable on all operating systems.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'America/Chicago'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# URL prefix for admin media -- CSS, JavaScript and images. Make sure to use a
# trailing slash.
# Examples: "http://foo.com/media/", "/media/".
ADMIN_MEDIA_PREFIX = '/media/'

# Make this unique, and don't share it with anybody.
SECRET_KEY = 'aadsfrewqsghf'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.load_template_source',
    'django.template.loaders.app_directories.load_template_source',
#     'django.template.loaders.eggs.load_template_source',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.middleware.doc.XViewMiddleware',
    'django.middleware.transaction.TransactionMiddleware',
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.admin',
    'djikiki',
    'registration',
)



#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

"""
WikiCreole to HTML converter
This program is an example of how the creole.py WikiCreole parser
can be used.


Copyright (c) 2007, Radomir Dopieralski <creole@sheep.art.pl>

All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

 * Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

 * Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in
   the documentation and/or other materials provided with the
   distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

import re
from creole import Parser

class Rules:
    # For the link targets:
    proto = r'http|https|ftp|nntp|news|mailto|telnet|file|irc'
    extern = r'(?P<extern_addr>(?P<extern_proto>%s):.*)' % proto
    interwiki = r'''
            (?P<inter_wiki> [A-Z][a-zA-Z]+ ) :
            (?P<inter_page> .* )
        '''

class HtmlEmitter:
    """
    Generate HTML output for the document
    tree consisting of DocNodes.
    """

    addr_re = re.compile('|'.join([
            Rules.extern,
            Rules.interwiki,
        ]), re.X | re.U) # for addresses

    def __init__(self, root):
        self.root = root

    def get_text(self, node):
        """Try to emit whatever text is in the node."""

        try:
            return node.children[0].content or ''
        except:
            return node.content or ''

    def html_escape(self, text):
        return text.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;')

    def attr_escape(self, text):
        return self.html_escape(text).replace('"', '&quot')

    # *_emit methods for emitting nodes of the document:

    def document_emit(self, node):
        return self.emit_children(node)

    def text_emit(self, node):
        return self.html_escape(node.content)

    def separator_emit(self, node):
        return u'<hr>';

    def paragraph_emit(self, node):
        return u'<p>%s</p>\n' % self.emit_children(node)

    def bullet_list_emit(self, node):
        return u'<ul>\n%s</ul>\n' % self.emit_children(node)

    def number_list_emit(self, node):
        return u'<ol>\n%s</ol>\n' % self.emit_children(node)

    def list_item_emit(self, node):
        return u'<li>%s</li>\n' % self.emit_children(node)

    def table_emit(self, node):
        return u'<table>\n%s</table>\n' % self.emit_children(node)

    def table_row_emit(self, node):
        return u'<tr>%s</tr>\n' % self.emit_children(node)

    def table_cell_emit(self, node):
        return u'<td>%s</td>' % self.emit_children(node)

    def table_head_emit(self, node):
        return u'<th>%s</th>' % self.emit_children(node)

    def emphasis_emit(self, node):
        return u'<i>%s</i>' % self.emit_children(node)

    def strong_emit(self, node):
        return u'<b>%s</b>' % self.emit_children(node)

    def header_emit(self, node):
        return u'<h%d>%s</h%d>\n' % (
            node.level, self.html_escape(node.content), node.level)

    def code_emit(self, node):
        return u'<tt>%s</tt>' % self.html_escape(node.content)

    def link_emit(self, node):
        target = node.content
        if node.children:
            inside = self.emit_children(node)
        else:
            inside = self.html_escape(target)
        m = self.addr_re.match(target)
        if m:
            if m.group('extern_addr'):
                return u'<a href="%s">%s</a>' % (
                    self.attr_escape(target), inside)
            elif m.group('inter_wiki'):
                raise NotImplementedError
        return u'<a href="%s">%s</a>' % (
            self.attr_escape(target), inside)

    def image_emit(self, node):
        target = node.content
        text = self.get_text(node)
        m = self.addr_re.match(target)
        if m:
            if m.group('extern_addr'):
                return u'<img src="%s" alt="%s">' % (
                    self.attr_escape(target), self.attr_escape(text))
            elif m.group('inter_wiki'):
                raise NotImplementedError
        return u'<img src="%s" alt="%s">' % (
            self.attr_escape(target), self.attr_escape(text))

    def macro_emit(self, node):
        raise NotImplementedError

    def break_emit(self, node):
        return u"<br>"

    def preformatted_emit(self, node):
        return u"<pre>%s</pre>" % self.html_escape(node.content)

    def default_emit(self, node):
        """Fallback function for emitting unknown nodes."""

        raise TypeError

    def emit_children(self, node):
        """Emit all the children of a node."""

        return u''.join([self.emit_node(child) for child in node.children])

    def emit_node(self, node):
        """Emit a single node."""

        emit = getattr(self, '%s_emit' % node.kind, self.default_emit)
        return emit(node)

    def emit(self):
        """Emit the document represented by self.root DOM tree."""

        return self.emit_node(self.root)

if __name__=="__main__":
    import sys
    document = Parser(unicode(sys.stdin.read(), 'utf-8', 'ignore')).parse()
    sys.stdout.write(HtmlEmitter(document).emit().encode('utf-8', 'ignore'))





from django.db import models
from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.auth.models import User
from django.forms import ValidationError
from django.db.models import permalink
from django.core.urlresolvers import reverse

import random
from creole import creole, creole2html

class Wiki(models.Model):
    """Stores site wide preferences"""
    name = models.TextField(max_length = 30)
    tagline = models.TextField(max_length = 100)

    def save (self):
        if Wiki.objects.all().count() and not self.id:
           raise Exception('Only one Wiki object allowed')
        super(Wiki, self).save()

    def __str__ (self):
        return self.name        
    
    class Meta:
        get_latest_by = 'id'

class Category(models.Model):
    """Category under which a page is classified. ManyToMany relationship with pages."""
    text = models.SlugField(max_length = 100, unique = True)

    def __str__ (self):
        return 'Category: %s' % self.text
    
    @models.permalink
    def get_absolute_url (self):
        return ('djikiki.views.category', [self.text])

class PageRevision(models.Model):
    """Stores a specific revision of the page"""
    wiki_text = models.TextField()
    html_text = models.TextField()
    edit_summary = models.TextField(max_length = 100)
    user = models.ForeignKey(User)
    is_discussion = models.BooleanField(default = False)
    created_on = models.DateTimeField(auto_now_add = 1)
    categories = models.ManyToManyField(Category)
    revision_for = models.ForeignKey('Page')
    revision_num = models.IntegerField(default = 0)
    
    def save (self): 
        document = creole.Parser(self.wiki_text).parse()
        self.html_text = creole2html.HtmlEmitter(document).emit()
        #self.html_text = creoleparser.creole2html(self.wiki_text)
        super(PageRevision, self).save() # Call the "real" save() method.

    def __str__ (self):
        return '%s-%s' % (self.edit_summary, self.wiki_text)
    
    @models.permalink
    def get_absolute_url (self):
        return ('djikiki.views.old_page', [self.id])

class Page(models.Model):
    """Stores the latest page, the page which needs to be displayed to the user."""
    title = models.SlugField(unique = True, max_length = 100)
    slug = models.SlugField(max_length = 100, unique = True)
    created_on = models.DateTimeField(auto_now_add = 1)
    modified_on = models.DateTimeField(auto_now_add = 1)
    current_revision = models.ForeignKey(PageRevision, related_name = 'main', blank = True, null = True)        
    current_disc_rev =  models.ForeignKey(PageRevision, related_name = 'discussion', blank = True, null = True) 
    user = models.ForeignKey(User)
    
    def save (self):
        if not self.slug:               
                self.slug = '-'.join(self.title.split())
                try:
                    Page.objects.get(slug = self.slug)
                except ObjectDoesNotExist, e:
                    pass
                else:
                   count = Page.objects.filter(slug__contains = self.slug).count()
                   self.slug = '%s-%s' % (self.slug, str(count + 1))
                   self.title = '%s-%s' % (self.title, str(count + 1))
        super(Page, self).save() # Call the "real" save() method    
    
    
    @models.permalink
    def get_absolute_url (self):
        return ('djikiki.views.detail', [self.slug])
    
    def get_discuss_url (self):
        return reverse('djikiki_discuss', args=[self.slug])
        
    @models.permalink
    def get_edit_url (self):
        return ('djikiki.views.edit', [self.slug])

    def get_discuss_edit_url (self):
        return reverse('djikiki_discuss_edit', args=[self.slug])
    
    @models.permalink
    def get_revision_url (self):
        return ('djikiki.views.revisions', [self.slug])

    def get_discuss_revision_url (self):
        return reverse('djikiki_discuss_revisions', args=[self.slug])

    def __str__ (self):
        return self.title

class FeaturedPage(models.Model):
    featured_on = models.DateField(auto_now_add = 1)
    page = models.ForeignKey(Page, unique = True)
    ordering = models.IntegerField(default = 0, unique = True)

    def save (self):
        count = FeaturedPage.objects.all().count()
        if count > 0:
            self.ordering = count + 1
        super(FeaturedPage, self).save()
        

    def __str__ (self):
        return '%s - %s' % (self.featured_on, self.page.title)

    class Meta:
        get_latest_by = '-ordering'
        


from setuptools import setup, find_packages

setup(
    name='django-djikiki',
    description='A django wiki',
    keywords='django, wiki',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    version="1.1",
    author="Agiliq Solutions",
    author_email="hello@agiliq.com",
    classifiers = ['Development Status :: 4 - Beta',
                   'Environment :: Web Environment',
                   'Framework :: Django',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: GNU General Public License (GPL)',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python',
                   'Topic :: Internet :: WWW/HTTP',
                   'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
                   'Topic :: Internet :: WWW/HTTP :: WSGI',
                   'Topic :: Software Development :: Libraries :: Application Frameworks',
                   'Topic :: Software Development :: Libraries :: Python Modules',
                   ],
    url="http://www.agiliq.com/",
    license="BSD",
    platforms=["all"],
)



from django.contrib import admin
from djikiki.models import Wiki, Category, PageRevision, Page, FeaturedPage


admin.site.register(Wiki)
admin.site.register(Category)
admin.site.register(PageRevision)
admin.site.register(Page)
admin.site.register(FeaturedPage)


from django.conf import settings
from django.http import Http404, HttpResponseRedirect
from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from models import Wiki

def handle404 (view_function):
    """If we are not in debug mode, convert ObjectDoesNotExist to Http404"""
    def wrapper (*args, **kwargs):
        if not settings.DEBUG:
            try:
                return view_function (*args, **kwargs)
            except ObjectDoesNotExist:
                raise Http404
        else:
            return view_function (*args, **kwargs)
    return wrapper

def check_wiki_installed(view_function):
    def is_install(*args, **kwargs):
        if Wiki.objects.all().count():
            return view_function(*args, **kwargs)
        else:
            return HttpResponseRedirect(reverse('djikiki_install'))
    return is_install